@@ -158,7 +158,7 @@ enum
     PROP_STATUS_MESSAGE
 };
 
-static guint lock_plug_signals [LAST_SIGNAL];
+static guint lock_plug_signals [LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (GSLockPlug, gs_lock_plug, GTK_TYPE_PLUG)
 
@@ -300,8 +300,7 @@ date_time_update (GSLockPlug *plug)
 	gchar *str;
 
 	datetime = g_date_time_new_now_local ();
-	/* Translators: Time format, see https://developer.gnome.org/glib/stable/glib-GDateTime.html#g-date-time-format */
-	time = g_date_time_format (datetime, _("%l:%M %p"));
+	time = g_date_time_format (datetime, "%X");
 	/* Translators: Date format, see https://developer.gnome.org/glib/stable/glib-GDateTime.html#g-date-time-format */
 	date = g_date_time_format (datetime, _("%A, %B %e"));
 
@@ -491,6 +490,10 @@ gs_lock_plug_get_text (GSLockPlug *plug,
 	{
 		*text = local_text;
 	}
+	else
+	{
+		g_free (local_text);
+	}
 }
 
 typedef struct
